@@ -64,7 +64,7 @@ def run_with(fname,query=True) :
           show=t.params.show_pics)
 
 def run_with_pdf(fname,**kwargs) :
-  pdf2txt(fname+".pdf")
+  pdf2txt(fname)
   run_with(fname, **kwargs)
 
 def chat_about(fname,qs=None) :
@@ -332,14 +332,40 @@ def answer_quest(q,talker) :
   sent_data, l2occ = db
   matches = defaultdict(set)
   nears = defaultdict(set)
-  answerer = Talker(from_text=q)
-  q_sent_data, q_l2occ = answerer.db
+
   unknowns = []
-  for j, q_lemma in enumerate(q_sent_data[0][LEMMA]):
-    q_tag = q_sent_data[0][TAG][j]
-    if q_tag[0] not in "NVJ": continue  # ppp(q_lemma,q_tag)
+  q_lemmas=[]
+  if talker.params.with_answerer:
+    answerer = Talker(from_text=q)
+    q_sent_data,_=answerer.db
+    for j, q_lemma in enumerate(q_sent_data[0][LEMMA]):
+      q_sent_data, q_l2occ = answerer.db
+      q_tag = q_sent_data[0][TAG][j]
+      if q_tag[0] not in "NVJ": continue  # ppp(q_lemma,q_tag)
+      q_lemmas.append((q_lemma,wn_tag(q_tag)))
+  else:
+    answerer = None
+    from nltk.tokenize import word_tokenize
+    from nltk.stem import WordNetLemmatizer
+    wnl = WordNetLemmatizer()
+    toks=word_tokenize(q)
+    tag=None
+    for t in toks:
+      tag='n'
+      l = wnl.lemmatize(t,tag)
+      if l==t :
+        tag='v'
+        l=wnl.lemmatize(t,tag)
+      if l==t :
+        tag='a'
+        l = wnl.lemmatize(t, tag)
+      l=l.lower()
+      q_lemmas.append((l,tag))
+
+  for q_lemma,wn_q_tag in q_lemmas:
     if not good_word(q_lemma) or q_lemma in ".?": continue
 
+    #  actual QA starts here
     ys = l2occ.get(q_lemma)
 
     if not ys:
@@ -348,7 +374,7 @@ def answer_quest(q,talker) :
       for sent, _pos in ys:
         matches[sent].add(q_lemma)
     if talker.params.expand_query > 0:
-      related = wn_all(talker.params.expand_query, 3, q_lemma, wn_tag(q_tag))
+      related = wn_all(talker.params.expand_query, 3, q_lemma, wn_q_tag)
       for r_lemma in related:
         if not good_word(q_lemma): continue
         zs = l2occ.get(r_lemma)
@@ -362,7 +388,7 @@ def answer_quest(q,talker) :
   if unknowns: tprint("UNKNOWNS:", unknowns, '\n')
 
   best = []
-  if talker.params.pers:
+  if talker.params.pers and talker.params.with_answerer:
     d = {x: r for x, r in answerer.pr.items() if good_word(x)}
     talker.pr = nx.pagerank(talker.g, personalization=d)
 
@@ -506,13 +532,18 @@ class Talker :
                from_json=None,
                from_file=None,
                from_text=None,
+               from_pdf=None,
                params=talk_params()
                ):
     '''creates data container from file or text document'''
     self.params=params
-
     self.from_file=from_file
-    if from_file:
+    if from_pdf:
+       from_pdf=from_pdf.replace('.pdf','')
+       pdf2txt(from_pdf)
+       self.from_file=from_pdf+".txt"
+       self.db = load(self.from_file, self.params.force)
+    elif from_file:
        self.db=load(from_file,self.params.force)
        self.from_file=from_file
     elif from_text :
@@ -856,7 +887,7 @@ class Talker :
     return g
 
   def dep_edge(self):
-    ''' dependency endge generator'''
+    ''' dependency edge generator'''
     sent_data, l2occ = self.db
     for info in sent_data:
       ws,ls,ts,_,deps,_=info
@@ -1181,13 +1212,42 @@ def take(k,g) :
     if i>=k : break
     yield x
 
-
-
 def pdf2txt(fname) :
   '''
     pdf to txt conversion with external tool - optional
+    make sure you install "poppler tools" for this to work!
   '''
-  subprocess.run(["pdftotext", fname])
+  subprocess.run(["pdftotext", fname+".pdf"])
+  clean_text_file(fname+".txt")
+
+def file2string(fname):
+  with open(fname, 'r') as f:
+    return f.read()
+
+def string2file(text,fname) :
+  with open(fname,'w') as g:
+    g.write(text)
+
+def clean_text_file(fname) :
+  print('cleaning: '+fname)
+  from nltk.tokenize import sent_tokenize, word_tokenize
+  data = file2string(fname)
+  texts=sent_tokenize(data)
+  clean=[]
+  for text in texts :
+    ws=word_tokenize(text)
+    good=0
+    bad=0
+    if len(ws)>256 : continue
+    for w in ws :
+      if w.isalpha() and len(w)>1 :good+=1
+      else : bad+=1
+    if good/(1+bad+good) <0.75 : continue
+    if ws[-1] not in ".?!" : ws.append(".")
+    sent=" ".join(ws)
+    clean.append(sent)
+  new_data="\n".join(clean)
+  string2file(new_data,fname)
 
 
 def path2fname(path) :
